package org.springblade.controller;

import java.util.List;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springblade.core.tool.api.R;
import org.springblade.db.exception.DataException;
import org.springblade.model.dto.ProvinceCrForm;
import org.springblade.model.po.ProvinceEntity;
import org.springblade.model.vo.ListVO;
import org.springblade.model.vo.ProvinceInfoVO;
import org.springblade.model.vo.ProvinceListVO;
import org.springblade.model.vo.ProvinceSelectListVO;
import org.springblade.service.ProvinceService;
import org.springblade.util.ListToTreeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 行政区划
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月27日 上午9:18
 */
@Api(tags = "行政区划", value = "Area")
@RestController
@RequestMapping("/paas/system/Area")
public class AreaController {

	@Autowired
	private ProvinceService provinceService;

	/**
	 * 列表（异步加载）
	 *
	 * @return
	 */
	@ApiOperation("列表（异步加载）")
	@GetMapping("/{nodeId}")
	public R<ListVO<ProvinceListVO>> list(@PathVariable("nodeId") String nodeId, String keyword) {
		List<ProvinceEntity> data = provinceService.getList(nodeId);
		List<ProvinceEntity> dataAll = data;
		if (StrUtil.isNotEmpty(keyword)) {
			data = data.stream()
					.filter(
							t -> t.getFullName().contains(keyword)
									|| t.getEnCode().contains(keyword))
					.collect(Collectors.toList());
		}
		List<ProvinceEntity> result = JSON.parseArray(JSON.toJSONString(ListToTreeUtil.treeWhere(data, dataAll)),
				ProvinceEntity.class);
		List<ProvinceListVO> treeList = JSON.parseArray(JSON.toJSONString(result), ProvinceListVO.class);
		int i = 0;
		for (ProvinceListVO entity : treeList) {
			boolean childNode = provinceService.getList(entity.getId()).size() <= 0;
			ProvinceListVO provinceListVO = JSON.parseObject(JSON.toJSONString(entity), ProvinceListVO.class);
			provinceListVO.setIsLeaf(childNode);
			provinceListVO.setHasChildren(childNode == true ? false : true);
			treeList.set(i, provinceListVO);
			i++;
		}
		ListVO<ProvinceListVO> vo = new ListVO<>();
		vo.setList(treeList);
		return R.data(vo);
	}

	/**
	 * 获取行政区划下拉框数据
	 *
	 * @return
	 */
	@ApiOperation("获取行政区划下拉框数据")
	@GetMapping("/{id}/Selector")
	public R<ListVO<ProvinceSelectListVO>> selectList(@PathVariable("id") String id) {
		List<ProvinceEntity> data = provinceService.getList(id);
		List<ProvinceSelectListVO> treeList = JSON.parseArray(JSON.toJSONString(data), ProvinceSelectListVO.class);
		int i = 0;
		for (ProvinceSelectListVO entity : treeList) {
			boolean childNode = provinceService.getList(entity.getId()).size() <= 0;
			ProvinceSelectListVO provinceListVO = JSON.parseObject(JSON.toJSONString(entity),
					ProvinceSelectListVO.class);
			provinceListVO.setIsLeaf(childNode);
			treeList.set(i, provinceListVO);
			i++;
		}
		ListVO<ProvinceSelectListVO> vo = new ListVO<>();
		vo.setList(treeList);
		return R.data(vo);
	}

	/**
	 * 信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取行政区划信息")
	@GetMapping("/{id}/Info")
	public R<ProvinceInfoVO> info(@PathVariable("id") String id) throws DataException {
		ProvinceEntity entity = provinceService.getById(id);
		ProvinceInfoVO vo = JSON.parseObject(JSON.toJSONString(entity), ProvinceInfoVO.class);
		if (!"-1".equals(entity.getParentId())) {
			ProvinceEntity parent = provinceService.getById(entity.getParentId());
			vo.setParentName(parent.getFullName());
		}
		return R.data(vo);
	}

	/**
	 * 新建
	 *
	 * @param provinceCrForm 实体对象
	 * @return
	 */
	@ApiOperation("添加行政区划")
	@PostMapping
	public R<String> create(@RequestBody @Valid ProvinceCrForm provinceCrForm) {
		ProvinceEntity entity = JSON.parseObject(JSON.toJSONString(provinceCrForm), ProvinceEntity.class);
		if (provinceService.isExistByEnCode(provinceCrForm.getEnCode(), entity.getId())) {
			return R.fail("区域编码不能重复");
		}
		if (StrUtil.isEmpty(provinceCrForm.getParentId())) {
			entity.setParentId("-1");
		}
		provinceService.save(entity);
		return R.data("新建成功");
	}

	/**
	 * 更新
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("修改行政区划")
	@PutMapping("/{id}")
	public R<String> update(@PathVariable("id") String id,
			@RequestBody @Valid ProvinceCrForm provinceCrForm) {
		ProvinceEntity entity = JSON.parseObject(JSON.toJSONString(provinceCrForm), ProvinceEntity.class);
		if (provinceService.isExistByEnCode(provinceCrForm.getEnCode(), id)) {
			return R.fail("区域编码不能重复");
		}
		boolean flag = provinceService.updateById(entity);
		if (flag == false) {
			return R.fail("更新失败，数据不存在");
		}
		return R.success("更新成功");
	}

	/**
	 * 删除
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("删除")
	@DeleteMapping("/{id}")
	public R<String> delete(@PathVariable("id") String id) {
		if (provinceService.getList(id).size() == 0) {
			ProvinceEntity entity = provinceService.getById(id);
			if (entity != null) {
				provinceService.removeById(id);
				return R.success("删除成功");
			}
			return R.fail("删除失败，数据不存在");
		} else {
			return R.fail("删除失败，当前有子节点数据");
		}
	}

	/**
	 * 更新行政区划状态
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("更新行政区划状态")
	@PutMapping("/{id}/Actions/State")
	public R<String> upState(@PathVariable("id") String id) {
		ProvinceEntity entity = provinceService.getById(id);
		if (entity.getEnabledMark() == null || "1".equals(String.valueOf(entity.getEnabledMark()))) {
			entity.setEnabledMark(0);
		} else {
			entity.setEnabledMark(1);
		}
		boolean flag = provinceService.updateById(entity);
		if (flag == false) {
			return R.fail("更新失败，数据不存在");
		}
		return R.success("更新成功");
	}

}
