package org.springblade.db.database.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.core.tool.api.R;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.model.RelationKey;
import org.springblade.db.cube.service.DynamicDataService;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * Oracle的关联表删除
 * 
 * @author William
 * @date 2023年1月8日 下午7:53:06
 */
@Slf4j
@Service
class DeleteActionOracle {
	@Autowired
	private DynamicDataService dynamicDataService;

	public R<String> deleteMultiTable(DataSourceModel ds, List<RelationBean> relationTree, List<String> idList) {
		LinkedHashMap<String, List<JSONObject>> allData = selectDataListWillDelete(null,
				ds,
				relationTree,
				idList,
				null);
		if (CollUtil.isEmpty(allData)) {
			return R.fail("数据不存在，删除失败");
		}

		IFly fly = IFly.get(ds);
		Iterator<String> iterator = allData.keySet().iterator();
		while (iterator.hasNext()) {
			String tableName = iterator.next();
			List<JSONObject> dataList = allData.get(tableName);
			if (CollUtil.isNotEmpty(dataList)) {
				// 该表存在要删除的数据
				Column primaryColumn = fly.getPrimaryColumn(tableName);
				// 存在主键，则可以删除，否则不允许删除
				if (primaryColumn == null) {
					continue;
				}

				List<String> primaryIdList = new ArrayList<>();
				for (JSONObject item : dataList) {
					String primaryId = item.get(primaryColumn.getColumnName()).toString();
					primaryIdList.add(primaryId);
				}
				deleteData(ds, tableName, primaryColumn, primaryIdList);
			}
		}
		return R.success("删除成功");
	}

	private void deleteData(DataSourceModel ds, String tableName, Column primaryColumn, List<String> idList) {
		StringBuffer sqlBuffer = new StringBuffer();
		String idsStr = idList.stream().collect(Collectors.joining(","));
		if (primaryColumn.getJavaType().equalsIgnoreCase("String")) {
			// 主键为字符串类型
			idsStr = idList.stream().collect(Collectors.joining("', '"));
			idsStr = "'" + idsStr + "'";
		}
		sqlBuffer.append("delete from " + tableName + " ");
		sqlBuffer.append("where " + primaryColumn.getColumnName() + " ");
		sqlBuffer.append("in (" + idsStr + ")");
		String sql = sqlBuffer.toString();
		log.info("delete sql is : " + sql);
		dynamicDataService.executeSql(ds, sql);
		log.info("delete finish from oracle");
	}

	/**
	 * 查询每个表将要删除的数据
	 * 
	 * @param ds              数据源
	 * @param relationTree    表关联关系树
	 * @param mainTableIdList 主表主键值
	 * @return
	 */
	public LinkedHashMap<String, List<JSONObject>> selectDataListWillDelete(
			LinkedHashMap<String, List<JSONObject>> allData, DataSourceModel ds,
			List<RelationBean> relationTree, List<String> mainTableIdList, List<JSONObject> parentDataList) {
		if (allData == null) {
			allData = new LinkedHashMap<>();
		}
		IFly fly = IFly.get(ds);
		for (RelationBean relation : relationTree) {
			List<JSONObject> dataList = new ArrayList<>();
			String relationTable = relation.getRelationTable();
			String tableName = relation.getTable();
			log.info("selectDataListWillDelete relation is " + JSON.toJSONString(relation));
			if (relationTable.equalsIgnoreCase(RelationUtil.ROOT_TABLE_PARENT)) {
				// 主表
				if (CollUtil.isEmpty(mainTableIdList)) {
					// 没有父表的主键则直接跳出递归
					log.info("selectDataListWillDelete main table have not primary key");
					return allData;
				}
				StringBuffer sqlBuffer = new StringBuffer();
				sqlBuffer.append("select * from " + tableName);
				sqlBuffer.append(" where 1=1 ");
				Column primaryColumn = fly.getPrimaryColumn(tableName);
				String idsStr = mainTableIdList.stream().collect(Collectors.joining(","));
				if (primaryColumn.getJavaType().equalsIgnoreCase("String")) {
					// 主键为字符串类型
					idsStr = mainTableIdList.stream().collect(Collectors.joining("', '"));
					idsStr = "'" + idsStr + "'";
				}
				sqlBuffer.append("and " + primaryColumn.getColumnName() + " ");
				sqlBuffer.append("in (" + idsStr + ")");
				String sql = sqlBuffer.toString();
				log.info("selectDataListWillDelete sql of main table is: " + sql);
				dataList = dynamicDataService.selectList(ds, sql);
				if (CollUtil.isNotEmpty(dataList)) {
					allData.put(tableName, dataList);
				}
			} else {
				// 子表
				if (CollUtil.isEmpty(parentDataList)) {
					// 没有父表的关联数据则直接跳出递归
					log.info("selectDataListWillDelete no parent table data");
					return allData;
				}
				List<RelationKey> relationKeyList = relation.getKeys();
				StringBuffer selectSql = new StringBuffer();
				selectSql.append("select * from " + relation.getTable());
				selectSql.append(" where 1=1 ");

				// 条件来源于上个表的关联字段数据
				List<Object> whereObjectList = new ArrayList<>();
				for (JSONObject map : parentDataList) {
					Iterator<String> iterator = map.keySet().iterator();
					for (RelationKey rk : relationKeyList) {
						String targetKey = rk.getTargetKey();

						RelationField field = RelationUtil.findByColumnName(targetKey, relation.getFields());
						if (field != null){
							if (field.getDataType().equalsIgnoreCase("datetime")) {
								selectSql.append(" and " + targetKey + " = to_date(?,'yyyy-mm-dd HH24:mi:ss')");
							} else {
								selectSql.append(" and " + targetKey + " = ?");
							}
						}
						String sourceKey = rk.getSourceKey();
						while (iterator.hasNext()) {
							String k = iterator.next();
							if (k.equalsIgnoreCase(sourceKey)) {
								Object v = map.get(k);
								whereObjectList.add(v);
							}
						}
					}

//					log.info("selectDataListWillDelete sql of child table is:" + selectSql.toString());
//					log.info("selectDataListWillDelete whereObjectList is:" + JSON.toJSONString(whereObjectList));
					Object[] objs = whereObjectList.toArray(new Object[whereObjectList.size()]);
					List<JSONObject> perList = fly.getJSONObjectList(selectSql.toString(), objs);
//					log.info("selectDataListWillDelete perList is: " + JSON.toJSONString(perList));
					if (CollUtil.isNotEmpty(perList)) {
						dataList.addAll(perList);
					}
				}

				if (CollUtil.isNotEmpty(dataList)) {
					allData.put(tableName, dataList);
				}
			}

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				// 递归继续查询
				return selectDataListWillDelete(allData, ds, relation.getChildren(), null, dataList);
			}
		}

		return allData;
	}

}
