package org.springblade.db.database.action;

import java.util.*;

import org.springblade.bi.client.model.DataSourceModel;
import org.springblade.config.JnpfKeyConsts;
import org.springblade.core.tool.support.Kv;
import org.springblade.db.cube.enums.DsType;
import org.springblade.db.cube.model.RelationBean;
import org.springblade.db.cube.model.RelationField;
import org.springblade.db.cube.model.RelationKey;
import org.springblade.db.cube.util.RelationUtil;
import org.springblade.db.model.UpdateBean;
import org.springblade.db.model.UpdateBean.UpdateColumn;
import org.springblade.db.sqlfly.config.Column;
import org.springblade.db.sqlfly.opt.IFly;
import org.springblade.util.SystemComponent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.collection.CollUtil;

/**
 * 
 * @author William
 * @date 2023年1月6日 下午4:19:35
 */
@Service
public class UpdateAction {

	@Autowired
	private SystemComponent sysCompoment;

	/**
	 * 递归构造更新对象
	 * 
	 * @param ds                  数据库连接
	 * @param dsType              数据库类型
	 * @param mainTableId         主表的主键
	 * @param formDataMap         前端提交的要更新的数据
	 * @param preTableOldDataList 上个表的旧数据
	 * @param preTableNewDataList 上个表的新数据
	 * @param relationTree        数据集关联关系的树形结构
	 * @param updateList          要构造的更新条件结构列表
	 */
	public void buildUpdate(DataSourceModel ds, DsType dsType, String mainTableId,
			Map<String, Object> formDataMap, List<Map<String, Object>> preTableOldDataList,
			List<UpdateColumn> preTableNewDataList, List<RelationBean> relationTree, List<UpdateBean> updateList,
			Kv fieldDefault) {
		for (RelationBean relation : relationTree) {
			UpdateBean updateBean = new UpdateBean(relation.getTable(), dsType);
			updateList.add(updateBean);

			// 当前表的新的数据
			List<UpdateColumn> newColumnList = new ArrayList<>();
			List<RelationField> fieldList = relation.getFields();
			for (int i = 0, size = fieldList.size(); i < size; i++) {
				// 构造当前表要更新的数据
				RelationField field = fieldList.get(i);
				String column = field.getField();
				String dataType = field.getDataType();
				Integer fieldId = field.getFieldId();
				Object value = null;
				// 前端是否输入的包含当前字段
				boolean isFromInput = formDataMap.containsKey(String.valueOf(fieldId));
				if (isFromInput) {
					value = formDataMap.get(String.valueOf(fieldId));
				}

				Kv fieldDefaultTemp = ((Kv) fieldDefault.get(String.valueOf(fieldId)));
				String jnpfKey = fieldDefaultTemp != null ? fieldDefaultTemp.getStr("jnpfKey") : "";
				if (sysCompoment.isSystemComponent(jnpfKey)) {
					// 系统默认组件，值由系统生成
					if (jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYTIME)
							|| jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYUSER)
							|| jnpfKey.equalsIgnoreCase(JnpfKeyConsts.MODIFYUSERID)) {
						// 字段是修改相关，则自动生成
						value = sysCompoment.getSystemValue(jnpfKey).toString();
					}
				}

				// 当前字段是否为关联字段，如果是关联字段则数据来自父表，并且忽略前端直接输入的内容
				boolean isRelationColumn = false;

				// 判断父表新数据
				if (CollUtil.isNotEmpty(preTableNewDataList)) {
					// 某个字段是否存在关联字段，如果存在,则从父表新数据中取值
					List<RelationKey> relationKeyList = relation.getKeys();
					for (RelationKey rk : relationKeyList) {
						String sourceKey = rk.getSourceKey();
						String targetKey = rk.getTargetKey();
						if (column.equalsIgnoreCase(targetKey)) {
							// 关联字段，值应该来源于父表
							isRelationColumn = true;
							for (UpdateColumn preNewData : preTableNewDataList) {
								if (preNewData.getColumn().equalsIgnoreCase(sourceKey)) {
									value = preNewData.getValue();
									break;
								}
							}
						}
					}
				}

				if (isFromInput || isRelationColumn) {
					// 该字段的值来自前端或者来自父表的关联字段，则更新，否则不更新
					UpdateColumn updateColumn = new UpdateBean.UpdateColumn(column, value, dataType);
					newColumnList.add(updateColumn);
					updateBean.addSetColumn(updateColumn);
				}
			}

			// 当前表的旧数据
			List<Map<String, Object>> curTableOldDataList = new ArrayList<>();
			IFly fly = IFly.get(ds);
			if (StrUtil.isNotBlank(mainTableId)) {
				// 首次循环，说明是主表
				Column primaryColumn = fly.getPrimaryColumn(relation.getTable());
				StringBuffer selectSql = new StringBuffer();
				selectSql.append("select * from " + relation.getTable());
				selectSql.append(" where " + primaryColumn.getColumnName() + "=?");
				Map<String, Object> rowData = fly.getMap(selectSql.toString(), mainTableId);
				curTableOldDataList.add(rowData);

				String dataType = null;
				if (primaryColumn.getJavaType().equalsIgnoreCase("String")) {
					dataType = "varchar";
				} else if (primaryColumn.getJavaType().equalsIgnoreCase("Date")) {
					dataType = "datetime";
				} else {
					dataType = "decimal";
				}
				// 当前表的更新条件字段信息
				UpdateColumn whereColumn = new UpdateColumn(primaryColumn.getColumnName(), mainTableId, dataType);
				updateBean.addWhereColumn(whereColumn);
			} else {
				// 不是主表，则根据条件查询旧数据
				if (CollUtil.isNotEmpty(preTableOldDataList)) {
					// 条件来源于上个表的关联字段数据
					List<String> whereColumnList = new ArrayList<>();
					List<Object> whereObjectList = new ArrayList<>();
					List<RelationKey> relationKeyList = relation.getKeys();
					StringBuffer selectSql = new StringBuffer();
					selectSql.append("select * from " + relation.getTable());
					selectSql.append(" where 1=1 ");
					for (RelationKey rk : relationKeyList) {
						String targetKey = rk.getTargetKey();
						selectSql.append(" and " + targetKey + " = ?");
						String sourceKey = rk.getSourceKey();
						for (Map<String, Object> map : preTableOldDataList) {
							Iterator<String> iterator = map.keySet().iterator();
							while (iterator.hasNext()) {
								String k = iterator.next();
								if (k.equalsIgnoreCase(sourceKey)) {
									Object v = map.get(k);
									whereObjectList.add(v);
									whereColumnList.add(targetKey);
								}
							}
						}
					}

					Object[] objs = whereObjectList.toArray(new Object[whereObjectList.size()]);
					if (objs.length>0 && objs[0] != null){
						List<Map<String, Object>> rowDataList = fly.getMapList(selectSql.toString(), objs);
						if (CollUtil.isNotEmpty(rowDataList)) {
							curTableOldDataList.addAll(rowDataList);
						}

						for (int n = 0, size = whereColumnList.size(); n < size; n++) {
							String columnName = whereColumnList.get(n);
							Object columnValue = whereObjectList.get(n);

							RelationField tempRf = RelationUtil.findByColumnName(columnName, fieldList);
							String dataType = tempRf != null ? tempRf.getDataType() : "varchar";
							UpdateColumn whereColumn = new UpdateColumn(columnName, columnValue, dataType);
							updateBean.addWhereColumn(whereColumn);
						}
					}


				}
			}

			if (CollUtil.isNotEmpty(relation.getChildren())) {
				// 有子表，则继续递归
				buildUpdate(ds,
						dsType,
						null,
						formDataMap,
						curTableOldDataList,
						newColumnList,
						relation.getChildren(),
						updateList,
						fieldDefault);
			}
		}
	}
}
